#include "hw_iface.h"
#include "hw_impl_zybo.h"
#include <stdint.h>
#include <string.h>
#include "type_def.h"

int px4flow_write(struct I2CDriver *i2c, u8 register_addr, u8 data);
int px4flow_read(struct I2CDriver *i2c, u8* recv_buffer, u8 register_addr, int size);

int zybo_optical_flow_reset(struct OpticalFlowDriver *self, px4flow_t *of) {
  memset(of, 0, sizeof(px4flow_t));

  return 0;
}

int zybo_optical_flow_read(struct OpticalFlowDriver *self, px4flow_t *of) {
  struct I2CDriver *i2c = self->i2c;
  int error = 0;

  struct {
    uint16_t frameCount;

    int16_t pixelFlowXSum;
    int16_t pixelFlowYSum;
    int16_t flowCompX;
    int16_t flowCompY;
    int16_t qual;

    int16_t gyroXRate;
    int16_t gyroYRate;
    int16_t gyroZRate;

    uint8_t gyroRange;
    uint8_t sonarTimestamp;
    int16_t groundDistance;
  } i2cFrame;

  u8 buf[sizeof(i2cFrame)];

  // Read the sensor value
  error = px4flow_read(i2c, buf, 0x00, sizeof(i2cFrame));

  if(error == 0) {
    //Copy into struct
    memcpy(&i2cFrame, buf, sizeof(i2cFrame));

    of->xVel = i2cFrame.flowCompX / 1000.;
    of->yVel = i2cFrame.flowCompY / 1000.;
    // They call it "sum", but it's just the latest pixel flow * 10
    of->flowX = (double)i2cFrame.pixelFlowXSum / 10.;
    of->flowY = (double)i2cFrame.pixelFlowYSum / 10.;
    of->quality = i2cFrame.qual;
    of->distance = i2cFrame.groundDistance / 1000.;
  }

  return error;
}

/////////////////////
// Helper functions
/////////////////////

int px4flow_write(struct I2CDriver *i2c, u8 register_addr, u8 data) {
	u8 buf[] = {register_addr, data};

	return i2c->write(i2c, PX4FLOW_DEVICE_ADDR, buf, 2);
}

int px4flow_read(struct I2CDriver *i2c, u8* recv_buffer, u8 register_addr, int size) {
	u8 buf[] = {register_addr};
	int error = 0;

	error = i2c->write(i2c, PX4FLOW_DEVICE_ADDR, buf, 1);
	if (error) return error;
	error = i2c->read(i2c, PX4FLOW_DEVICE_ADDR, recv_buffer, size);
	return error;
}

